import { Knex } from 'knex';
import { groups, tasks, steps, images, users, notes, printHistory } from '../db';
import type { Printer } from '@shared/index';

interface Context {
  db: Knex;
  printer: Printer;
}

export const resolvers = {
  Query: {
    groups: async (_: any, __: any, { db }: Context) => {
      return await groups(db).select('*');
    },
    group: async (_: any, { id }: { id: string }, { db }: Context) => {
      const group = await groups(db).where('id', id).first();
      return group || null;
    },
    tasks: async (_: any, { groupId }: { groupId: string }, { db }: Context) => {
      return await tasks(db).where('group_id', groupId).select('*');
    },
    task: async (_: any, { id }: { id: string }, { db }: Context) => {
      const task = await tasks(db).where('id', id).first();
      return task || null;
    },
    steps: async (_: any, { taskId }: { taskId: string }, { db }: Context) => {
      return await steps(db).where('task_id', taskId).orderBy('order').select('*');
    },
    step: async (_: any, { id }: { id: string }, { db }: Context) => {
      const step = await steps(db).where('id', id).first();
      return step || null;
    },
    recentTasks: async (_: any, __: any, { db }: Context) => {
      return await tasks(db)
        .orderBy('created_at', 'desc')
        .limit(10)
        .select('*');
    },
    frequentTasks: async (_: any, __: any, { db }: Context) => {
      return await tasks(db)
        .orderBy('created_at', 'desc')
        .limit(10)
        .select('*');
    },
    users: async (_: any, __: any, { db }: Context) => {
      return await users(db).select('*');
    },
    user: async (_: any, { id }: { id: string }, { db }: Context) => {
      const user = await users(db).where('id', id).first();
      return user || null;
    },
    notes: async (_: any, { stepId, taskId }: { stepId?: string; taskId?: string }, { db }: Context) => {
      const query = notes(db);
      if (stepId) {
        query.where('step_id', stepId);
      }
      if (taskId) {
        query.where('task_id', taskId);
      }
      return await query.select('*');
    },
    printHistory: async (_: any, { taskId, stepId }: { taskId?: string; stepId?: string }, { db }: Context) => {
      const query = printHistory(db);
      if (taskId) {
        query.where('task_id', taskId);
      }
      if (stepId) {
        query.where('step_id', stepId);
      }
      return await query.select('*');
    },
  },

  Mutation: {
    createGroup: async (_: any, { name, parentId }: { name: string; parentId?: string }, { db }: Context) => {
      let parent_id: number | undefined = undefined;
      if (parentId) {
        const parent = await groups(db).where('id', parentId).first();
        if (!parent) throw new Error('Parent group not found');
        parent_id = parseInt(parentId);
      }
      const [id] = await groups(db).insert({ name, parent_id });
      return await groups(db).where('id', id).first();
    },
    createTask: async (_: any, { name, groupId }: { name: string; groupId: string }, { db }: Context) => {
      const group = await groups(db).where('id', groupId).first();
      if (!group) throw new Error('Group not found');
      const [id] = await tasks(db).insert({
        name,
        group_id: parseInt(groupId),
        print_count: 0,
      });
      return await tasks(db).where('id', id).first();
    },
    createStep: async (_: any, { name, instructions, taskId, order }: { name: string; instructions: string; taskId: string; order: number }, { db }: Context) => {
      const task = await tasks(db).where('id', taskId).first();
      if (!task) throw new Error('Task not found');
      const [id] = await steps(db).insert({
        name,
        instructions,
        task_id: parseInt(taskId),
        order,
        print_count: 0,
      });
      return await steps(db).where('id', id).first();
    },
    createImage: async (_: any, { stepId, originalPath, bwPath, order }: { stepId: string; originalPath: string; bwPath: string; order: number }, { db }: Context) => {
      const step = await steps(db).where('id', stepId).first();
      if (!step) throw new Error('Step not found');
      const [id] = await images(db).insert({
        step_id: parseInt(stepId),
        original_path: originalPath,
        bw_path: bwPath,
        order,
      });
      return await images(db).where('id', id).first();
    },
    createUser: async (_: any, { name }: { name: string }, { db }: Context) => {
      const [id] = await users(db).insert({ name });
      return await users(db).where('id', id).first();
    },
    createNote: async (
      _: any,
      { content, stepId, taskId, userId }: { content: string; stepId?: string; taskId?: string; userId: string },
      { db }: Context
    ) => {
      const user = await users(db).where('id', userId).first();
      if (!user) throw new Error('User not found');
      if (stepId) {
        const step = await steps(db).where('id', stepId).first();
        if (!step) throw new Error('Step not found');
      } else if (taskId) {
        const task = await tasks(db).where('id', taskId).first();
        if (!task) throw new Error('Task not found');
      }
      const [id] = await notes(db).insert({
        content,
        step_id: stepId ? parseInt(stepId) : undefined,
        task_id: taskId ? parseInt(taskId) : undefined,
        created_by: parseInt(userId),
      });
      return await notes(db).where('id', id).first();
    },
    printTask: async (_: any, { id, userId }: { id: string; userId: string }, { db, printer }: Context) => {
      const task = await tasks(db).where('id', id).first();
      if (!task) throw new Error('Task not found');
      const user = await users(db).where('id', userId).first();
      if (!user) throw new Error('User not found');
      await printer.printTask(task, db);
      await printHistory(db).insert({
        user_id: parseInt(userId),
        task_id: parseInt(id),
        printed_at: new Date(),
        created_at: new Date(),
        updated_at: new Date(),
      });
      await tasks(db).where('id', id).update({
        print_count: (task.print_count || 0) + 1,
        last_printed_at: new Date(),
        updated_at: new Date(),
      });
      return await tasks(db).where('id', id).first();
    },
    printStep: async (_: any, { id, userId }: { id: string; userId: string }, { db, printer }: Context) => {
      const step = await steps(db).where('id', id).first();
      if (!step) throw new Error('Step not found');
      const user = await users(db).where('id', userId).first();
      if (!user) throw new Error('User not found');
      await printer.printStep(step, db);
      await printHistory(db).insert({
        user_id: parseInt(userId),
        step_id: parseInt(id),
        printed_at: new Date(),
        created_at: new Date(),
        updated_at: new Date(),
      });
      await steps(db).where('id', id).update({
        print_count: (step.print_count || 0) + 1,
        last_printed_at: new Date(),
        updated_at: new Date(),
      });
      return await steps(db).where('id', id).first();
    },
  },

  Group: {
    tasks: async (group: { id: number }, _: any, { db }: Context) => {
      return await tasks(db).where('group_id', group.id).select('*');
    },
  },

  Task: {
    group: async (task: { group_id: number }, _: any, { db }: Context) => {
      const group = await groups(db).where('id', task.group_id).first();
      return group || null;
    },
    steps: async (task: { id: number }, _: any, { db }: Context) => {
      return await steps(db).where('task_id', task.id).orderBy('order').select('*');
    },
    notes: async (task: { id: number }, _: any, { db }: Context) => {
      return await notes(db).where('task_id', task.id).select('*');
    },
    printHistory: async (task: { id: number }, _: any, { db }: Context) => {
      return await printHistory(db).where('task_id', task.id).select('*');
    },
  },

  Step: {
    task: async (step: { task_id: number }, _: any, { db }: Context) => {
      const task = await tasks(db).where('id', step.task_id).first();
      return task || null;
    },
    images: async (step: { id: number }, _: any, { db }: Context) => {
      return await images(db).where('step_id', step.id).orderBy('order').select('*');
    },
    notes: async (step: { id: number }, _: any, { db }: Context) => {
      return await notes(db).where('step_id', step.id).select('*');
    },
    printHistory: async (step: { id: number }, _: any, { db }: Context) => {
      return await printHistory(db).where('step_id', step.id).select('*');
    },
  },

  Note: {
    createdBy: async (note: { created_by: number }, _: any, { db }: Context) => {
      const user = await users(db).where('id', note.created_by).first();
      return user || null;
    },
  },

  PrintHistory: {
    user: async (history: { user_id: number }, _: any, { db }: Context) => {
      const user = await users(db).where('id', history.user_id).first();
      return user || null;
    },
    task: async (history: { task_id?: number }, _: any, { db }: Context) => {
      if (!history.task_id) return null;
      const task = await tasks(db).where('id', history.task_id).first();
      return task || null;
    },
    step: async (history: { step_id?: number }, _: any, { db }: Context) => {
      if (!history.step_id) return null;
      const step = await steps(db).where('id', history.step_id).first();
      return step || null;
    },
  },
};