import { testDb } from './setup';
import { groups, tasks, steps, images } from '../index';
import { Group, Task, Step, Image } from '../types';

describe('Database Operations', () => {
  beforeEach(async () => {
    // Clean up the database before each test
    await images(testDb).del();
    await steps(testDb).del();
    await tasks(testDb).del();
    await groups(testDb).del();
  });

  afterAll(async () => {
    // Close the database connection after all tests
    await testDb.destroy();
  });

  describe('Groups', () => {
    it('should create and retrieve a group', async () => {
      const groupData = {
        name: 'Test Group',
        parent_id: null,
      };

      const [id] = await groups(testDb).insert(groupData);
      const group = await groups(testDb).where('id', id).first();

      expect(group).toMatchObject({
        id,
        name: groupData.name,
        parent_id: groupData.parent_id,
      });
    });

    it('should create a nested group', async () => {
      const [parentId] = await groups(testDb).insert({ name: 'Parent Group' });
      const [childId] = await groups(testDb).insert({
        name: 'Child Group',
        parent_id: parentId,
      });

      const retrievedChild = await groups(testDb).where('id', childId).first();
      expect(retrievedChild?.parent_id).toBe(parentId);
    });
  });

  describe('Tasks', () => {
    it('should create and retrieve a task', async () => {
      const [groupId] = await groups(testDb).insert({ name: 'Test Group' });
      const taskData = {
        title: 'Test Task',
        group_id: groupId,
        print_count: 0,
      };

      const [id] = await tasks(testDb).insert(taskData);
      const task = await tasks(testDb).where('id', id).first();

      expect(task).toMatchObject({
        id,
        title: taskData.title,
        group_id: taskData.group_id,
        print_count: taskData.print_count,
      });
    });
  });

  describe('Steps', () => {
    it('should create and retrieve a step', async () => {
      const [groupId] = await groups(testDb).insert({ name: 'Test Group' });
      const [taskId] = await tasks(testDb).insert({
        title: 'Test Task',
        group_id: groupId,
      });

      const stepData = {
        title: 'Test Step',
        instructions: 'Test Instructions',
        task_id: taskId,
        order: 1,
        print_count: 0,
      };

      const [id] = await steps(testDb).insert(stepData);
      const step = await steps(testDb).where('id', id).first();

      expect(step).toMatchObject({
        id,
        title: stepData.title,
        instructions: stepData.instructions,
        task_id: stepData.task_id,
        order: stepData.order,
        print_count: stepData.print_count,
      });
    });
  });

  describe('Images', () => {
    it('should create and retrieve an image', async () => {
      const [groupId] = await groups(testDb).insert({ name: 'Test Group' });
      const [taskId] = await tasks(testDb).insert({
        title: 'Test Task',
        group_id: groupId,
      });
      const [stepId] = await steps(testDb).insert({
        title: 'Test Step',
        instructions: 'Test Instructions',
        task_id: taskId,
        order: 1,
      });

      const imageData = {
        step_id: stepId,
        original_path: '/path/to/original.png',
        bw_path: '/path/to/bw.png',
        order: 1,
      };

      const [id] = await images(testDb).insert(imageData);
      const image = await images(testDb).where('id', id).first();

      expect(image).toMatchObject({
        id,
        step_id: imageData.step_id,
        original_path: imageData.original_path,
        bw_path: imageData.bw_path,
        order: imageData.order,
      });
    });
  });
});