import fs from 'fs/promises';
import path from 'path';
import { Task, Step } from '../db/types';
import { steps } from '../db';
import { Knex } from 'knex';
import logger from '../logger';
import { Printer } from './index';

export class TestPrinter implements Printer {
  private readonly outputDir: string;

  constructor() {
    this.outputDir = path.join(process.cwd(), 'test-output');
    this.ensureOutputDir();
  }

  private async ensureOutputDir() {
    try {
      await fs.mkdir(this.outputDir, { recursive: true });
    } catch (error) {
      logger.error('Failed to create output directory:', error);
    }
  }

  async getTaskSteps(db: Knex, task: Task): Promise<Step[]> {
    return await steps(db).where('task_id', task.id).orderBy('order').select('*');
  }

  async printTask(task: Task, db: Knex): Promise<void> {
    const taskSteps = await this.getTaskSteps(db, task);
    const timestamp = new Date().toISOString().replace(/[:.]/g, '-');
    const filename = path.join(this.outputDir, `task-${task.id}-${timestamp}.txt`);

    const content = [
      `Task: ${task.title}`,
      '='.repeat(40),
      '',
      ...taskSteps.map((step, index) => [
        `Step ${index + 1}: ${step.title}`,
        '-'.repeat(40),
        step.instructions,
        '',
      ]).flat(),
    ].join('\n');

    await fs.writeFile(filename, content);
    logger.info(`Printed task ${task.id} to ${filename}`);
  }

  async printStep(step: Step, db: Knex): Promise<void> {
    void db; // avoid unused parameter warning
    const timestamp = new Date().toISOString().replace(/[:.]/g, '-');
    const filename = path.join(this.outputDir, `step-${step.id}-${timestamp}.txt`);

    const content = [
      `Step: ${step.title}`,
      '='.repeat(40),
      '',
      step.instructions,
      '',
    ].join('\n');

    await fs.writeFile(filename, content);
    logger.info(`Printed step ${step.id} to ${filename}`);
  }
}